/**
 * This is a minimal config.
 *
 * If you need the full config, get it from here:
 * https://unpkg.com/browse/tailwindcss@latest/stubs/defaultConfig.stub.js
 */

module.exports = {
	content: [
		/**
		 * HTML. Paths to Django template files that will contain Tailwind CSS classes.
		 */

		/*  Templates within theme app (<tailwind_app_name>/templates), e.g. base.html. */
		"../templates/**/*.html",

		/*
		 * Main templates directory of the project (BASE_DIR/templates).
		 * Adjust the following line to match your project structure.
		 */
		"../../templates/**/*.html",

		/*
		 * Templates in other django apps (BASE_DIR/<any_app_name>/templates).
		 * Adjust the following line to match your project structure.
		 */
		"../../**/templates/**/*.html",

		/**
		 * JS: If you use Tailwind CSS in JavaScript, uncomment the following lines and make sure
		 * patterns match your project structure.
		 */
		/* JS 1: Ignore any JavaScript in node_modules folder. */
		// '!../../**/node_modules',
		/* JS 2: Process all JavaScript files in the project. */
		// '../../**/*.js',

		/**
		 * Python: If you use Tailwind CSS classes in Python, uncomment the following line
		 * and make sure the pattern below matches your project structure.
		 */
		// '../../**/*.py'
	],
	theme: {
		container: {
			center: true,
			padding: {
				DEFAULT: "2rem",
				xl: "4rem",
			},
		},
		extend: {
			fontFamily: {
				nerd: ["DejaVuSansMono"],
			},
			colors: {
				bluelight: {
					light: "#378ec880",
					DEFAULT: "#317eb1",
					dark: "#006bb380",
					translucent: {
						light: "#317eb140",
						DEFAULT: "#317eb180",
						dark: "#2facff40",
					},
					background: {
						light: "#317eb1",
						DEFAULT: "#26638c",
						dark: "#1b4764",
					},
					link: "hsl(203.91, 56.64%, 75%)",
				},
				"trans-pride": {
					cyan: {
						DEFAULT: "#5adefc",
						translucent: "#5adefc69",
					},
					white: {
						DEFAULT: "#eeeeeeff",
						translucent: "#eeeeee69",
					},
					pink: {
						DEFAULT: "#ff8cda",
						translucent: "#ff8cda69",
					},
				},
				highlight: {
					checks: {
						text: "hsl(210, 92.86%, 90%)",
						light: "hsla(293.9,72.3%,45%,1)",
						DEFAULT: "#A431B1",
						dark: "hsla(293.9,72.3%,30%,1)",
					},
					entrances: {
						text: "hsla(41.86,95.56%,90%,1)",
						light: "hsla(23.9,85%,40%,1)",
						DEFAULT: "#B16431",
						dark: "hsla(23.9,85%,25%,1)",
					},
					both: {
						text: "hsla(116.4,75.76%,90%,1)",
						light: "hsla(113.9,72.3%,40%,1)",
						DEFAULT: "#3EB131",
						dark: "hsla(113.9,72.3%,25%,1)",
					},
					undiscovered: {
						text: "hsla(0,0%,96%,1)",
						light: "hsla(0,0%,45%,1)",
						dark: "hsla(0,0%,20%,1)",
					},
					empty: {
						text: "hsla(0,100%,100%,1)",
						light: "hsla(0,100%,100%,0.05)",
						dark: "hsla(0,100%,100%,0.17)",
					},
				},
				"holy-cross": {
					DEFAULT: "fffae0",
				},
			},
		},
	},
	plugins: [
		/**
		 * '@tailwindcss/forms' is the forms plugin that provides a minimal styling
		 * for forms. If you don't like it or have own styling for forms,
		 * comment the line below to disable '@tailwindcss/forms'.
		 */
		require("@tailwindcss/forms"),
		require("@tailwindcss/typography"),
		require("@tailwindcss/aspect-ratio"),
		require("tailwind-scrollbar"),
	],
};
